//
//  CLBTableViewSectionController.m
//  CLBTableViewSectionController
//
//  Created by Philipp Steinacher on 28.01.11.
//

#import "CLBTableViewSectionController.h"

@interface CLBTableViewSectionController (hidden)

- (NSInteger)sectionIndexForIdentifier:(NSString *)identifier;

@end

@implementation CLBTableViewSectionController
@synthesize theTableView, delegate, shouldAdaptToSizeOfAllCells;

- (id)initWithTableView:(UITableView *)tableView
{
    self = [super init];
    
    if (self != nil) {
        theTableView = tableView;
        sections = [NSMutableArray array];
        sectionNames = [NSMutableArray array];
    }
    
    return self;
}

- (NSInteger)sectionIndexForIdentifier:(NSString *)identifier
{
    return [sectionNames indexOfObject:identifier];
}

#pragma mark - Adding Sections

- (void)addSection:(CLBTableViewSection *)section withIdentifier:(NSString *)identfier
{
    [sections addObject:section];
    [sectionNames addObject:identfier];
    
    [theTableView reloadData];
}

- (void)insertSection:(CLBTableViewSection *)section atIndex:(NSInteger)index withIdentifier:(NSString *)identifier
{
    [sections insertObject:section atIndex:index];
    [sectionNames insertObject:identifier atIndex:index];
}

- (void)insertSection:(CLBTableViewSection *)section beforeSection:(NSString *)sectionIdentifier withIdentifier:(NSString *)identifier
{
    NSInteger indexForIdentifier = [self sectionIndexForIdentifier:identifier];
    
    [sections insertObject:sections atIndex:indexForIdentifier];
    [sectionNames insertObject:identifier atIndex:indexForIdentifier];
}

- (void)insertSection:(CLBTableViewSection *)section afterSection:(NSString *)sectionIdentifier withIdentifier:(NSString *)identifier
{
    NSInteger indexForIdentifier = [self sectionIndexForIdentifier:identifier];
    
    [sections insertObject:sections atIndex:indexForIdentifier+1];
    [sectionNames insertObject:identifier atIndex:indexForIdentifier+1];
}

#pragma mark - Removing sections

- (void)removeAllSections
{
    [sections removeAllObjects];
    [sectionNames removeAllObjects];
}

- (void)removeSectionAtIndex:(NSInteger)index
{
    [sections removeObjectAtIndex:index];
    [sectionNames removeObjectAtIndex:index];
}

- (void)removeSectionWithIndentifier:(NSString *)identifier
{
    NSInteger indexForIdentifier = [self sectionIndexForIdentifier:identifier];
    
    [sections removeObjectAtIndex:indexForIdentifier];
    [sectionNames removeObjectAtIndex:indexForIdentifier];
}

- (void)removeSection:(CLBTableViewSection *)section
{
    NSInteger sectionIndex = [sectionNames indexOfObject:section];
    
    if (sectionIndex != NSNotFound) {
        [sections removeObjectAtIndex:sectionIndex];
        [sectionNames removeObjectAtIndex:sectionIndex];
    }
}

#pragma mark - Section getter

- (CLBTableViewSection *)sectionAtIndex:(NSInteger)index
{
    return [sections objectAtIndex:index];
}

- (CLBTableViewSection *)sectionWithIdentifier:(NSString *)identifier
{
    NSInteger indexForIdentifier = [self sectionIndexForIdentifier:identifier];
    CLBTableViewSection *section;
    
    if (indexForIdentifier != NSNotFound) {
        section = [sections objectAtIndex:indexForIdentifier];
    }
    
    return section;
}

#pragma mark - Section controlls

- (NSArray *)allSectionIdentifier
{
    return sectionNames;
}

- (NSArray *)allSections
{
    return sections;
}

- (NSInteger)numberOfRowsInTableView
{
    __block NSInteger numberOfRows = 0;
    
    [sections enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        int numberOfSectionForRow = [self tableView:theTableView numberOfRowsInSection:idx];
        numberOfRows += numberOfSectionForRow;
    }];
        
    return numberOfRows;
}

- (NSInteger)numberOfSectionsInTableView
{
    return [sections count];
}

#pragma mark - Shorthands

- (void)registerTableViewCell:(UITableViewCell *)cell inSectionWithIdentifier :(NSString *)identifier andHeight:(NSInteger)height
{
    CLBTableViewSection *section = [self sectionWithIdentifier:identifier];
    
    if (section != nil) {
        [section registerRowWithBlock:^UITableViewCell *(UITableView *tableView) {
            UITableViewCell *dequeuedCell = [tableView dequeueReusableCellWithIdentifier:cell.reuseIdentifier];
            UITableViewCell *toBeReturnedCell;
            
            if (dequeuedCell == nil || dequeuedCell != cell) {
                toBeReturnedCell = cell;
            } else {
                toBeReturnedCell = dequeuedCell;
            }
            
            return nil;   
        } andHeight:height];
    }
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [sections count];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    CLBTableViewSection *tableViewSection = [sections objectAtIndex:section];
    return [tableViewSection title];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    CLBTableViewSection *tableViewSection = [sections objectAtIndex:section];
    return [tableViewSection numberOfRows];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    CLBTableViewSection *tableViewSection = [sections objectAtIndex:indexPath.section];
    return [tableViewSection cellHeightForIndex:indexPath.row];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    CLBTableViewSection *tableViewSection = [sections objectAtIndex:indexPath.section];
    return [tableViewSection cellForRow:indexPath.row inTableView:tableView];
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)_tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    CLBTableViewSection *tableViewSection = [sections objectAtIndex:indexPath.section];
    return [tableViewSection didSelectCellAtRow:indexPath.row]; 
}

@end
